searchData={"items":[{"type":"module","title":"Mutex","doc":"This is the main module in this application, it implements a mutex as a\nGenServer with a notification system to be able to await lock releases.\n\nSee [`README.md`](https://hexdocs.pm/mutex/readme.html) for usage\ninstructions.","ref":"Mutex.html"},{"type":"function","title":"Mutex.await/3","doc":"Locks a key if it is available, or waits for the key to be freed before\nattempting again to lock it.\n\nReturns the lock or fails with a timeout.\n\nDue to the notification system, multiple attempts can be made to lock if\nmultiple processes are competing for the key.\n\nSo timeout will be *at least* for the passed amount of milliseconds, but may\nbe slightly longer.\n\nDefault timeout is `5000` milliseconds. If the timeout is reached, the caller\nprocess exits as in `GenServer.call/3`.  More information in the\n[timeouts](https://hexdocs.pm/elixir/GenServer.html#call/3-timeouts) section.","ref":"Mutex.html#await/3"},{"type":"function","title":"Mutex.await_all/2","doc":"Awaits multiple keys at once. Returns once all the keys have been locked,\ntimeout is `:infinity`.\n\nIf two processes are trying to lock `[:user_1, :user_2]` and\n`[:user_2, :user_3]` at the same time, this function ensures that no deadlock\ncan happen and that one process will eventually lock all the keys.\n\nMore information at the end of the\n[deadlocks section](https://hexdocs.pm/mutex/readme.html#avoiding-deadlocks).","ref":"Mutex.html#await_all/2"},{"type":"function","title":"Mutex.child_spec/1","doc":"Returns a specification to start a mutex under a supervisor.\n\nSee the \"Child specification\" section in the `Supervisor` module for more\ndetailed information.","ref":"Mutex.html#child_spec/1"},{"type":"function","title":"Mutex.give_away/4","doc":"Sets the the process identified by `pid` as the new owner of the `lock`.\n\nIf succesful, that new owner will be sent a `{:\"MUTEX-TRANSFER\", from_pid,\nlock, gift_data} ` message. If it is not alive, the lock will be released.\n\nThis function only supports single key locks.","ref":"Mutex.html#give_away/4"},{"type":"function","title":"Mutex.goodbye/1","doc":"Tells the mutex to release *all* the keys owned by the calling process and\nreturns immediately with `:ok`.","ref":"Mutex.html#goodbye/1"},{"type":"function","title":"Mutex.lock/2","doc":"Attemps to lock a resource on the mutex and returns immediately with the\nresult, which is either a `Mutex.Lock` structure or `{:error, :busy}`.","ref":"Mutex.html#lock/2"},{"type":"function","title":"Mutex.lock!/2","doc":"Attemps to lock a resource on the mutex and returns immediately with the lock\nor raises an exception if the key is already locked.","ref":"Mutex.html#lock!/2"},{"type":"function","title":"Mutex.release/2","doc":"Releases the given lock synchronously.","ref":"Mutex.html#release/2"},{"type":"function","title":"Mutex.release_async/2","doc":"Tells the mutex to free the given lock and immediately returns `:ok` without\nwaiting for the actual release.\n\nUnlike `release/2`, this function will not raise if the calling process is\nnot lock owner. In that case, the lock is not released and an error is\nlogged.\n\nSupports only single key locks.","ref":"Mutex.html#release_async/2"},{"type":"function","title":"Mutex.start/1","doc":"Starts a mutex with no process linking. Given options are passed as options\nfor a `GenServer`, it's a good place to set the name for registering the\nprocess.\n\nSee `start_link/1` for options.","ref":"Mutex.html#start/1"},{"type":"function","title":"Mutex.start_link/1","doc":"Starts a mutex linked to the calling process.\n\nAccepts only `t:GenServer.options()`.\n\nSee [`GenServer`\noptions](https://hexdocs.pm/elixir/GenServer.html#t:options/0).","ref":"Mutex.html#start_link/1"},{"type":"function","title":"Mutex.under/4","doc":"Alias for `with_lock/4`.","ref":"Mutex.html#under/4"},{"type":"function","title":"Mutex.under_all/3","doc":"Alias for `with_lock/3`.","ref":"Mutex.html#under_all/3"},{"type":"function","title":"Mutex.with_lock/4","doc":"Awaits a lock for the given key, executes the given fun and releases\nthe lock immediately.\n\nIf an exeption is raised or thrown in the fun, the lock is\nautomatically released.\n\nIf a function of arity 1 is given, it will be passed the lock.\nOtherwise the arity must be 0. You should not manually release the\nlock within the function.","ref":"Mutex.html#with_lock/4"},{"type":"function","title":"Mutex.with_lock_all/3","doc":"Awaits a lock for the given keys, executes the given fun and\nreleases the lock immediately.\n\nIf an exeption is raised or thrown in the fun, the lock is\nautomatically released.\n\nIf a function of arity 1 is given, it will be passed the lock.\nOtherwise the arity must be 0. You should not manually release the\nlock within the function.","ref":"Mutex.html#with_lock_all/3"},{"type":"type","title":"Mutex.key/0","doc":"A key can be any term.","ref":"Mutex.html#t:key/0"},{"type":"type","title":"Mutex.name/0","doc":"Identifier for a mutex process.","ref":"Mutex.html#t:name/0"},{"type":"module","title":"Mutex.Lock","doc":"This module defines a struct containing the key(s) locked with all the\nlocking functions in `Mutex`.","ref":"Mutex.Lock.html"},{"type":"type","title":"Mutex.Lock.t/0","doc":"The struct containing the key(s) locked during a lock operation. `:type`\nspecifies wether there is one or more keys.","ref":"Mutex.Lock.html#t:t/0"},{"type":"extras","title":"Overview","doc":"# A simple mutex for Elixir\n\n[![Module Version](https://img.shields.io/hexpm/v/mutex.svg)](https://hex.pm/packages/mutex)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/mutex/)\n[![License](https://img.shields.io/hexpm/l/mutex.svg)](https://github.com/lud/mutex/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/lud/mutex.svg)](https://github.com/lud/mutex/commits/main)\n\n`Mutex` is a lightweight mutex or \"locks\" implementation that fits under your\nsupervision tree and allows processes to work on shared ressources one by one.\n\nThis can be a simple alternative to job queues or filesystem transactions and\nallows to limit resource usage on a server.\n\nAlso, `Mutex` supports locking multiple keys without deadlocks.\n\n\n- [Documentation](#documentation)\n- [Installation](#installation)\n- [Basic usage](#basic-usage)\n- [Error Handling](#error-handling)\n- [Avoiding Deadlocks](#avoiding-deadlocks)\n- [Lock handover](#lock-handover)\n- [Name registration](#name-registration)\n- [Metadata (Removed)](#metadata-removed)\n- [Copyright and License](#copyright-and-license)\n\n\n## [Documentation](https://hexdocs.pm/mutex/)\n\nThe documentation is hosted on [Hexdocs](https://hexdocs.pm/mutex/).","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"This package can be installed by adding `:mutex` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mutex, \"~> 3.0\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Basic usage - Overview","doc":"This library implements a mutex as a process that you would generally start\nunder a supervision tree.\n\nSee [start_link/1](https://hexdocs.pm/mutex/Mutex.html#start_link/1) for options.\n\n```elixir\ndefmodule MyApp.Application do\n  @moduledoc false\n\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      {Mutex, name: MyApp.Mutex}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n  end\nend\n```\n\nNow from anywhere in the application code you can lock access to a a resource\nwith the guarantee that no concurrent execution of that code can be executed.\n\nOf course, this requires that all code attempting to access that resource uses\nthe mutex as well.\n\n```elixir\nlock = Mutex.await(MyApp.Mutex, :file_manager)\ndo_something_with_the_filesystem(arg)\n:ok = Mutex.release(MyApp.Mutex, lock)\n```\n\nTo automatically lock and release the key, use `Mutex.with_lock/3`:\n\n```elixir\nMutex.with_lock(MyApp.Mutex, :file_manager, fn ->\n  do_something_with_the_filesystem(arg)\nend)\n```\n\nHere is a more complete example of working with storing data in a file:\n\n```elixir\npath = \"/tmp/counter-#{System.system_time(:microsecond)}\"\n\nFile.write!(path, \"0\")\n\ntasks =\n  for _ <- 1..10 do\n    Task.async(fn ->\n      Mutex.with_lock(MyApp.Mutex, :file_manager, fn ->\n        counter = String.to_integer(File.read!(path))\n        File.write!(path, Integer.to_string(counter + 1))\n      end)\n    end)\n  end\n\nEnum.each(tasks, &Task.await/1)\n\ncounter = String.to_integer(File.read!(path))\nIO.puts(\"Total count is: #{counter}\")\n```\n\nThis will output:\n\n```\nTotal count is: 10\n```\n\nThe same code but without the mutex will generally give a final value of `1`.","ref":"readme.html#basic-usage"},{"type":"extras","title":"Error Handling - Overview","doc":"Whenever a process that locked a key on the mutex crashes, the mutex\nautomatically unlocks the key so any other process can lock it in its turn.\n\nBut if you catch exceptions you may forget to release the keys and keep\nunnecessary keys locked for a while :\n\n```elixir\n# Do not do this\ntry do\n  lock = Mutex.await(MyApp.Mutex, :some_key)\n  throw(:fail)\n  # This will never be called:\n  Mutex.release(MyApp.Mutex, lock)\ncatch\n  :throw, :fail -> :ok\nend\n```\n\nWhenever possible, avoid to lock keys in `try`, `if`, `for`, ... blocks.\n\nWhen using `Mutex.with_lock/3`, the lock will be automatically released if the\ngiven fun raises, throws, or exits.\n\nExceptions and thrown values are reraised and rethrown so you still have to\nhandle them.\n\n```elixir\n# Do this instead\ntry do\n  Mutex.with_lock(MyApp.Mutex, :some_key, fn ->\n    throw(:fail)\n  end)\ncatch\n  :throw, :fail -> :ok\nend\n```\n\nA multilock version is also available with `Mutex.with_lock_all/3`.\n\nBoth functions can accept a fun of arity `1` that will be given the lock as its\nargument.","ref":"readme.html#error-handling"},{"type":"extras","title":"Avoiding Deadlocks - Overview","doc":"A deadlock would occur if multiple processes would attempt to lock the same keys\nin a different order:\n\n```elixir\n  # Do not do this\n\n  def handle_order(buyer, seller) do\n    lock1 = Mutex.await(MyApp.Mutex, buyer)\n    lock2 = Mutex.await(MyApp.Mutex, seller)\n    do_some_work_with_users(buyer, seller)\n    Mutex.release(MyApp.Mutex, lock1)\n    Mutex.release(MyApp.Mutex, lock2)\n  end\n\n  spawn(fn -> handler_order(:user_1, :user_2) end) # Process 1\n  spawn(fn -> handler_order(:user_2, :user_1) end) # Process 2\n```\n\nProcess 1 will first lock `:user_1` and process 2 will lock `:user_2`, and then\neach process is waiting for the key that is already locked by the other one.\n\n**If any process should have, at any given time, several keys locked, those keys\nshall have been locked all at once.**\n\nThis simple rule is enough to be free from deadlocks, and `Mutex.await_all/2` is\nthe simplest way to respect that rule.\n\n```elixir\n# Do this instead\n\ndef handle_order(buyer, seller) do\n  lock = Mutex.await_all(MyApp.Mutex, [buyer, seller])\n  do_some_work_with_users(buyer, seller)\n  Mutex.release(MyApp.Mutex, lock)\nend\n```\n\nIf you really have to lock keys in a loop, or at different places, the\n`Mutex.goodbye/1` function allows to simply release all the keys locked by the\ncalling process at once.","ref":"readme.html#avoiding-deadlocks"},{"type":"extras","title":"Lock handover - Overview","doc":"It is possible to transfer ownership of a lock to another process with the\n`Mutex.give_away/4` function.\n\nThis is useful if you need to check that a resource is available and then start\na long running process that needs the lock and terminate quickly.\n\nFor instance, we want to have one instance only of a video encoding process in\nour application. Video encoding is started from a web controller that needs to\nsend an HTTP response in a timely fashion.\n\n```elixir\ndef handle_request(encoding_request) do\n  case Mutex.lock(MyApp.Mutex, :encoding_server) do\n    {:error, :busy} ->\n      \"encoding not available\"\n\n    {:ok, lock} ->\n      {:ok, task_pid} =\n        Task.Supervisor.start_child(\n          Encoding.Supervisor,\n          &encode_video/0\n        )\n\n      # Here we pass the video path as the \"gift data\" but in the real world\n      # that should be given as an argument to the task function.\n      :ok = Mutex.give_away(MyApp.Mutex, lock, task_pid, encoding_request.path)\n      \"encoding started\"\n  end\nend\n\ndef encode_video do\n  receive do\n    {:\"MUTEX-TRANSFER\", _, lock, video_path} ->\n      Encoder.encode_video(video_path)\n      Mutex.release(MyApp.Mutex, lock)\n      Encoder.cleanup(video_path)\n  after\n    1000 -> exit(:no_lock_received)\n  end\nend\n```","ref":"readme.html#lock-handover"},{"type":"extras","title":"Name registration - Overview","doc":"Processes having a key locked in a `Mutex` can be called using a `:via` tuple with `GenServer.call/3` or `GenServer.cast/2`.\n\nThe `:via` tuple is built as follows:\n\n```elixir\n{\n  :via,\n  Mutex,\n  { ,  }\n}\n```\n\nExample calling a `GenServer` process with such tuples:\n\n```elixir\n{:ok, _} = Mutex.start_link(name: MyApp.Mutex)\n\ndefmodule MyGenServer do\n  def init(_), do: Mutex.lock(MyApp.Mutex, :some_key)\n  def handle_call(:greetings, _, state), do: {:reply, :hello, state}\nend\n\n{:ok, _} = GenServer.start_link(MyGenServer, [])\n:hello = GenServer.call({:via, Mutex, {MyApp.Mutex, :some_key}}, :greetings)\n```\n\nThis lets you check if a key is currently locked in the Mutex:\n\n```\n# Returns #PID<…> or nil\nGenServer.whereis({:via, Mutex, {MyApp.Mutex, :some_key}})\n```\n\nAs any other name registry, you can directly set the name of your process in `GenServer.start_link/3`:\n\n```elixir\nGenServer.start_link(MyGenServer, [], name: {:via, Mutex, {MyApp.Mutex, :some_key}})\n```\n\nIn this case the lock is automatically taken by the `GenServer` process.\n\n**Important**, this mechanism is intended for use with mutexes handling a small\namount of processes working together. Do not use name registration with a single\nmutex in you application or you could get bottlenecks. The\n[Registry](https://hexdocs.pm/elixir/Registry.html#module-using-in-via) module\nis intended for such uses and supports unique keys and partitionning registered\nkeys for performance.","ref":"readme.html#name-registration"},{"type":"extras","title":"Metadata (Removed) - Overview","doc":"A Mutex process could carry metadata attached to it and would send it to any\nprocess acquiring a lock. This has been removed as it was confusing. Metadata\nwas not tied to a given key but to the mutex itself.\n\nTo replace this functionality, you may use a `Registry`, an ETS table,\n`:persistent_term` or other shared data mechanisms provided by the Elixir and\nErlang platforms.","ref":"readme.html#metadata-removed"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2017, Ludovic Demblans\n\nThis work is free. You can redistribute it and/or modify it under the\nterms of the MIT License. See the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"License","doc":"# The MIT License\n\nCopyright (c) 2017, Ludovic Demblans\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of\nthis software and associated documentation files (the \"Software\"), to deal in\nthe Software without restriction, including without limitation the rights to\nuse, copy, modify, merge, publish, distribute, sublicense, and/or sell copies\nof the Software, and to permit persons to whom the Software is furnished to do\nso, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}